define(['appStorage', 'browser', 'events', 'require'], function (appStorage, browser, events, require) {

    'use strict';

    console.log = function () { };

    function sharePolyfill(data) {

        return new Promise(function (resolve, reject) {
            MainActivity.share(data.text, data.title, data.imageUrl, data.url);
            resolve();
        });
    }

    function getDeviceProfile() {

        return require(['./exoprofilebuilder']).then(function (responses) {
            return responses[0].buildProfile({});
        });
    }

    if (navigator.connection && navigator.connection.addEventListener) {
        navigator.connection.addEventListener('change', updateDownlink);
    }

    function updateDownlink() {

        var max = AndroidAppHost.getDownlink();

        if (!max) {
            // there must have been a problem determining this
            return;
        }

        try {
            navigator.connection.downlink = max;

            return;
        }
        catch (err) {
            console.log('error setting downlink: ' + err);
        }

        try {
            Object.defineProperty(navigator.connection, 'downlink', {
                value: max,
                //writable: false
            });
        }
        catch (err) {
            console.log('error in defineProperty downlink: ' + err);
        }
    }

    var deviceId;
    var deviceName;
    var appName;
    var appVersion;

    var supportedFeatures = function () {

        var features = {};

        if (AndroidAppHost.supportsSync()) {
            features.sync = true;
        }
        features.customsyncpath = true;

        if (AndroidAppHost.hasCamera()) {
            features.cameraupload = true;
        }

        if (AndroidAppHost.supportsDisplayModeSwitching()) {
            features.changerefreshrate = true;
        }

        if (AndroidAppHost.getSDKVersion() >= 21) {
            features.sharing = true;
        }

        features.exit = true;
        features.exitmenu = true;
        features.htmlaudioautoplay = true;
        features.htmlvideoautoplay = true;
        features.externallinks = true;
        features.externalappinfo = true;
        features.externallinkdisplay = true;
        features.targetblank = true;
        features.multiserver = true;
        features.youtube = true;
        features.backgroundvideo = true;

        features.physicalbackbutton = true;

        features.physicalvolumecontrol = true;
        features.nativevolumeosd = true;

        features.remotecontrol = true;

        features.castmenuhashchange = true;

        features.otherapppromotions = true;
        features.externalplayerintent = true;

        features.displaylanguage = true;
        features.fileinput = true;

        features.chromecast = true;

        if (AndroidAppHost.getSDKVersion() >= 21) {
            features.subtitleappearancesettings = true;
        }

        features.connectsignup = true;
        features.restrictedplugins = true;
        features.serversetup = true;

        if (!AndroidAppHost.isTvDevice()) {
            features.displaymode = true;
        }

        if (self.SpeechRecognition && AndroidAppHost.supportsSpeechRecognition()) {
            features.speechrecognition = true;
        }

        return features;
    }();

    var speechRecognitionPromiseResolve;
    var speechRecognitionPromiseReject;

    var appHost = {
        getWindowState: function () {
            return document.windowState || 'Normal';
        },
        setWindowState: function (state) {
            throw new Error('setWindowState is not supported and should not be called');
        },
        exit: function () {

            MainActivity.exitApp();
        },
        supports: function (command) {
            return supportedFeatures[command];
        },
        preferVisualCards: true,
        moreIcon: 'dots-vert',
        getSyncProfile: getDeviceProfile,

        init: function () {

            // Remove special characters
            deviceName = AndroidAppHost.getDeviceName();

            deviceId = appStorage.getItem('legacyDeviceId') || AndroidAppHost.getDeviceId();
            appVersion = AndroidAppHost.getAppVersion();
            appName = AndroidAppHost.getAppName();

            return Promise.resolve();
        },

        deviceName: function () {
            return deviceName;
        },

        deviceId: function () {
            return deviceId;
        },

        appName: function () {
            return appName;
        },

        appVersion: function () {
            return appVersion;
        },

        getPushTokenInfo: function () {

            var info = {};

            var fcmToken = AndroidAppHost.getFcmToken();
            if (fcmToken) {
                info.PushToken = fcmToken;
                info.PushTokenType = "firebase";
            }

            return info;
        },

        setTheme: function (themeSettings) {

            AndroidAppHost.setStatusBarForegroundColor(themeSettings.androidStatusBarForegroundColor);
            AndroidAppHost.setNavigationBarForegroundColor(themeSettings.androidNavigationBarForegroundColor);
        },

        setUserScalable: function (scalable) {

            if (browser.tv) {
                return;
            }

            var att = scalable ?
                'viewport-fit=cover, width=device-width, initial-scale=1, minimum-scale=1, user-scalable=yes' :
                'viewport-fit=cover, width=device-width, initial-scale=1, minimum-scale=1, maximum-scale=1, user-scalable=no';

            document.querySelector('meta[name=viewport]').setAttribute('content', att);
        },

        deviceIconUrl: function () {
            return 'https://github.com/MediaBrowser/Emby.Resources/raw/master/images/devices/android.png';
        },

        getDefaultLayout: function () {
            return AndroidAppHost.getDefaultLayout();
        },

        requestSpeechRecognitionPermission: function () {

            var result = MainActivity.authorizeSpeechRecognition();

            if (result) {
                return Promise.resolve();
            }

            return new Promise(function (resolve, reject) {

                speechRecognitionPromiseResolve = resolve;
                speechRecognitionPromiseReject = reject;
            });
        }
    };

    if (!navigator.share) {

        navigator.share = sharePolyfill;
    }

    //document.addEventListener('viewshow', function (e) {

    //    MainActivity.setPullToRefreshEnabled(e.detail.canRefresh === true);
    //});

    appHost.setSystemUIHidden = function (hidden) {
        MainActivity.setFullscreen(hidden);
    };

    function normalizeSystemPixel(value) {

        var ratio = window.devicePixelRatio;
        if (ratio) {
            value = Math.floor(value / ratio);
        }

        return value;
    }

    var windowInsets = {
        top: 0,
        left: 0,
        right: 0,
        bottom: 0
    };

    window.onSpeechRecognitionPermissionResult = function (granted) {

        let fn = granted ? speechRecognitionPromiseResolve : speechRecognitionPromiseReject;
        if (fn) {
            fn();
        }

        speechRecognitionPromiseResolve = null;
        speechRecognitionPromiseReject = null;
    };

    window.onWindowInsetChanged = function (top, right, bottom, left) {

        top = normalizeSystemPixel(top);
        right = normalizeSystemPixel(right);
        bottom = normalizeSystemPixel(bottom);
        left = normalizeSystemPixel(left);

        windowInsets = {
            top: top,
            left: left,
            right: right,
            bottom: bottom
        };

        top = top + 'px';
        right = right + 'px';
        bottom = bottom + 'px';
        left = left + 'px';

        try {

            var style = document.documentElement.style;

            style.setProperty('--window-inset-bottom', bottom);
            style.setProperty('--window-inset-top', top);
            style.setProperty('--window-inset-right', right);
            style.setProperty('--window-inset-left', left);
        }
        catch (err) {

            // shouldn't happen, but handle potential older webviews that don't support this
            console.log('error in style.setProperty: ' + err);
        }
    };

    if (!screen.orientation) {
        screen.orientation = {};
    }

    screen.orientation.lock = function (mode) {

        MainActivity.lockLandscapeOrientation();
        return Promise.resolve();
    };

    screen.orientation.unlock = function () {
        MainActivity.unlockOrientation();
    };

    if (self.MainActivity) {
        MainActivity.requestApplyInsets();

        document.addEventListener('appready', function () {
            MainActivity.onAppReady();

            updateDownlink();
        });
    }

    document.addEventListener('viewshow', function viewDidLoad() {
        document.removeEventListener('viewshow', viewDidLoad);
        MainActivity.onViewShow();
    });

    var _isHidden = false;
    function onAppVisible() {

        if (_isHidden) {
            _isHidden = false;
            //console.log('triggering app resume event');
            events.trigger(appHost, 'resume');
        }
    }

    function onAppHidden() {

        if (!_isHidden) {
            _isHidden = true;
            //console.log('app is hidden');
        }
    }

    document.addEventListener('visibilitychange', function () {

        if (document.hidden) {
            onAppHidden();
        } else {
            onAppVisible();
        }
    });

    if (self.addEventListener) {
        self.addEventListener('focus', onAppVisible);
        self.addEventListener('blur', onAppHidden);
    }

    document.addEventListener('focus', function (e) {

        if (!self.MainActivity) {
            return;
        }

        var tagName = e.target.tagName;

        switch (tagName) {
            case 'INPUT':
            case 'TEXTAREA':
                MainActivity.setInputHasFocus(true);
                break;
            default:
                MainActivity.setInputHasFocus(false);
                break;
        }

    }, true);

    return appHost;
});