define(['appSettings', 'connectionManager'], function (appSettings, connectionManager) {
    'use strict';

    function buildProfile(options) {

        var profile = {};

        // Only setting MaxStaticBitrate for older servers
        profile.MaxStreamingBitrate = profile.MaxStaticBitrate = 200000000;
        profile.MusicStreamingTranscodingBitrate = 192000;
        profile.MaxStaticMusicBitrate = appSettings.maxStaticMusicBitrate();

        profile.DirectPlayProfiles = [];

        var supportsHevc = CodecCapabilities.supportsHevc();

        profile.DirectPlayProfiles.push({
            Type: 'Video',
            VideoCodec: supportsHevc ? null : '-hevc'
        });

        profile.DirectPlayProfiles.push({
            Type: 'Audio',
            Container: null
        });

        profile.TranscodingProfiles = [];

        var hlsVideoCodecs = ['h264', 'mpeg2video'];
        if (supportsHevc) {
            hlsVideoCodecs.push('hevc');
        }

        var item = options.item;

        var apiClient = item ? connectionManager.getApiClient(item) : null;
        var serverSupportsfmp4Hls = apiClient ? apiClient.isMinServerVersion('4.7.0.16') : false;

        var hlsVideoSegmentContainers = [];

        if (serverSupportsfmp4Hls) {
            hlsVideoSegmentContainers.push('m4s');
        }
        hlsVideoSegmentContainers.push('ts');

        profile.TranscodingProfiles.push({
            Container: hlsVideoSegmentContainers.join(','),
            Type: 'Video',
            AudioCodec: 'ac3,aac,mp3',
            VideoCodec: hlsVideoCodecs.join(','),
            Context: 'Streaming',
            Protocol: 'hls',
            MaxAudioChannels: '6',
            MinSegments: '1',
            BreakOnNonKeyFrames: true,
            SegmentLength: '3',
            ManifestSubtitles: 'vtt'
        });

        if (!item || !AndroidVlcPlayer.streamAndSaveEnabled(item.ServerId)) {
            profile.TranscodingProfiles.push({

                Container: 'ts',
                Type: 'Audio',
                AudioCodec: 'aac',
                Context: 'Streaming',
                Protocol: 'hls',
                MinSegments: '1',
                SegmentLength: '3',
                BreakOnNonKeyFrames: true
            });
        }

        profile.TranscodingProfiles.push({
            Container: 'mp3',
            Type: 'Audio',
            AudioCodec: 'mp3',
            Context: 'Streaming',
            Protocol: 'http'
        });

        profile.TranscodingProfiles.push({
            Container: 'mkv',
            Type: 'Video',
            AudioCodec: 'aac,mp3,ac3',
            VideoCodec: 'h264',
            Context: 'Static',
            MaxAudioChannels: '2'
        });

        profile.TranscodingProfiles.push({
            Container: 'mp3',
            Type: 'Audio',
            AudioCodec: 'mp3',
            Context: 'Static',
            Protocol: 'http',
            MaxAudioChannels: '2'
        });

        profile.ContainerProfiles = [];

        profile.CodecProfiles = [];

        if (supportsHevc) {

            var hevcProfiles = ['Main', 'Main 10', 'Rext'];

            profile.CodecProfiles.push({
                Type: 'Video',
                Codec: 'hevc',
                Conditions: [
                    {
                        Condition: 'EqualsAny',
                        Property: 'VideoProfile',
                        Value: hevcProfiles.join('|'),
                        IsRequired: false
                    }]
            });
        }

        profile.CodecProfiles.push({
            Type: 'Video',
            Codec: 'h264',
            Conditions: [
                {
                    Condition: 'LessThanEqual',
                    Property: 'VideoLevel',
                    Value: CodecCapabilities.getMaxH264Level(),
                    IsRequired: false
                }]
        });

        // Subtitle profiles
        // External vtt or burn in
        profile.SubtitleProfiles = [];
        profile.SubtitleProfiles.push({
            Format: 'vtt',
            Method: 'Hls'
        });
        profile.SubtitleProfiles.push({
            Format: 'srt',
            Method: 'External'
        });
        profile.SubtitleProfiles.push({
            Format: 'ssa',
            Method: 'External'
        });
        profile.SubtitleProfiles.push({
            Format: 'ass',
            Method: 'External'
        });
        profile.SubtitleProfiles.push({
            Format: 'smi',
            Method: 'External'
        });
        profile.SubtitleProfiles.push({
            Format: 'srt',
            Method: 'Embed'
        });
        profile.SubtitleProfiles.push({
            Format: 'subrip',
            Method: 'Embed'
        });
        profile.SubtitleProfiles.push({
            Format: 'ass',
            Method: 'Embed'
        });
        profile.SubtitleProfiles.push({
            Format: 'ssa',
            Method: 'Embed'
        });
        profile.SubtitleProfiles.push({
            Format: 'dvb_teletext',
            Method: 'Embed'
        });
        profile.SubtitleProfiles.push({
            Format: 'dvb_subtitle',
            Method: 'Embed'
        });
        profile.SubtitleProfiles.push({
            Format: 'dvbsub',
            Method: 'Embed'
        });
        profile.SubtitleProfiles.push({
            Format: 'pgs',
            Method: 'Embed'
        });
        profile.SubtitleProfiles.push({
            Format: 'pgssub',
            Method: 'Embed'
        });
        profile.SubtitleProfiles.push({
            Format: 'dvdsub',
            Method: 'Embed'
        });
        profile.SubtitleProfiles.push({
            Format: 'vtt',
            Method: 'Embed'
        });
        profile.SubtitleProfiles.push({
            Format: 'sub',
            Method: 'Embed'
        });
        profile.SubtitleProfiles.push({
            Format: 'idx',
            Method: 'Embed'
        });
        profile.SubtitleProfiles.push({
            Format: 'smi',
            Method: 'Embed'
        });
        profile.SubtitleProfiles.push({
            Format: 'mov_text',
            Method: 'Embed'
        });
        profile.SubtitleProfiles.push({
            Format: 'eia_608',
            Method: 'Embed'
        });
        profile.SubtitleProfiles.push({
            Format: 'eia_708',
            Method: 'Embed'
        });
        profile.SubtitleProfiles.push({
            Format: 'eia_608',
            Method: 'VideoSideData'
        });
        profile.SubtitleProfiles.push({
            Format: 'eia_708',
            Method: 'VideoSideData'
        });

        profile.ResponseProfiles = [];

        return profile;
    }

    return {
        buildProfile: buildProfile
    };
});